<?xml version="1.0"?>
<!-- 
   - The contents of this file are subject to the Mozilla Public
   - License Version 1.0 (the "License"); you may not use this file
   - except in compliance with the License. You may obtain a copy of
   - the License at http://www.mozilla.org/MPL/
   - 
   - Software distributed under the License is distributed on an "AS
   - IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
   - implied. See the License for the specific language governing
   - rights and limitations under the License.
   - 
   - The Original Code is XML Schema Compiler (Preprocessor).
   - 
   - The Initial Developer of the Original Code is AEA Technology - Engineering Software.
   -  Portions created by AEA Technology - Engineering Software are
   - Copyright (C) 2000 AEA Technology - Engineering Software.  All
   - Rights Reserved.
   - 
   - Contributor(s): Curt Arnold <curta@hyprotech.com>
  --> 

<xsl:transform xmlns:xsl='http://www.w3.org/1999/XSL/Transform' 
     version="1.0"
	 xmlns:xsd="http://www.w3.org/1999/XMLSchema"
	 xmlns:xsc="http://XSDComp.sourceforge.net/Namespaces/2000/XMLSchema-compiled">
<xsl:output method="xml" version="1.0" indent="yes"/>


<!--  document root template  -->
<xsl:template match="/">
	<!--  terminate processing if document element is not schema 
				otherwise process templates  -->
	<xsl:choose>
		<xsl:when test="not(xsd:schema)">
			<xsl:message terminate="yes">Fatal: Top level element in source is [tagname] not schema.</xsl:message>
		</xsl:when>
		<xsl:otherwise>
			<!--  apply default mode template to schema  -->
			<xsc:package>
				<!--  any implied schema   -->
				<xsl:for-each select="xsd:schema">
					<xsl:call-template name="implicit-import">
						<xsl:with-param name="namespace">http://www.w3.org/XML/1998/namespace</xsl:with-param>
						<xsl:with-param name="schemaLocation">xml.xsd</xsl:with-param>
					</xsl:call-template>

					<!--  this is to get the datatypes, I'd much rather the schema definition namespace 
					              imported the datatypes namespace   -->
					<xsl:call-template name="implicit-import">
						<xsl:with-param name="namespace">http://www.w3.org/1999/XMLSchema</xsl:with-param>
						<xsl:with-param name="schemaLocation">XMLSchema.xsd</xsl:with-param>
					</xsl:call-template>

					<xsl:call-template name="implicit-import">
						<xsl:with-param name="namespace">http://www.w3.org/1999/XMLSchema-instance</xsl:with-param>
						<xsl:with-param name="schemaLocation">XMLSchema-instance.xsd</xsl:with-param>
					</xsl:call-template>
				</xsl:for-each>


				<xsl:apply-templates mode="imports"/>


				<xsl:apply-templates/>
			</xsc:package>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<!--  suppress almost everything in imports mode   -->
<xsl:template match="*|@*|text()" mode="imports"/>

<!--  supress the has-facet and has-property that appears in the datatypes inclusion  -->
<xsl:template match="xsd:appinfo">
	<xsd:appinfo>
		<xsl:for-each select="*">
			<xsl:choose>
				<xsl:when test="self::xsd:has-facet"/>
				<xsl:when test="self::xsd:has-property"/>
				<xsl:otherwise>
					<xsl:apply-templates/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsd:appinfo>
</xsl:template>


<xsl:template match="xsd:schema" mode="imports">
	<xsl:apply-templates select="xsd:import" mode="imports"/>
</xsl:template>
	

<xsl:template match="xsd:schema">
	<xsc:schema>
		<!-- copy all attributes except targetPrefix   -->
		<xsl:apply-templates select="@targetNamespace"/>
		<xsl:apply-templates select="@version"/>

		<xsl:variable name="annotations" select="xsd:annotation"/>
		<xsl:if test="$annotations">
			<xsd:annotation>
				<xsl:apply-templates select="$annotations/*"/>
			</xsd:annotation>
		</xsl:if>

		<!--   express targetNamespace attribute of xsc:namespace element -->
		<xsl:choose>
			<!--  no target namespace throw error   -->
			<xsl:when test="not(@targetNamespace)">
				<xsl:message terminate="yes">Fatal Error: No namespace specified for schema.</xsl:message>
			</xsl:when>

			<!--  generate namespace element       -->
			<xsl:otherwise>
				<xsc:namespace uri="{@targetNamespace}">
					<!--  find target prefix in appinfo child    -->
					<xsl:variable name="targetPrefix" select="@xsc:prefix"/>
					<xsl:if test="@version"><xsl:attribute name="schemaVersion"><xsl:value-of select="@version"/></xsl:attribute></xsl:if>
					<!--  prefix specified, add to namespace element   -->
					<xsl:choose>
						<xsl:when test="$targetPrefix">
							<xsc:prefix><xsl:value-of select="$targetPrefix"/></xsc:prefix>
						</xsl:when>

						<xsl:otherwise>
							<xsl:message>Information: schema for namespace <xsl:value-of select="@targetNamespace"/> does not have a xsc:prefix attribute which could result in qualified references to not be resolved.</xsl:message>
						</xsl:otherwise>
					</xsl:choose>
				</xsc:namespace>
			</xsl:otherwise>
		</xsl:choose>													

		<xsl:apply-templates select="xsd:import"/>

		<!--  add namespace prefix definitions for xml and xmlns   -->
		<xsc:namespace uri="http://www.w3.org/XML/1998/namespace">
			<xsc:prefix>xml</xsc:prefix>
			<!-- KLUDGE:  used in part2.xsd    -->
			<xsc:prefix>x</xsc:prefix>
		</xsc:namespace>

		<!--  add namespace prefix definitions for xml and xmlns   -->
		<xsc:namespace uri="http://www.w3.org/1999/XMLSchema-instance">
			<xsc:prefix>xsi</xsc:prefix>
		</xsc:namespace>
	

		<xsl:apply-templates select="xsd:include"/>
		<xsl:apply-templates select="xsd:notation"/>
		<xsl:apply-templates select="xsd:simpleType"/>

		<xsl:apply-templates select="xsd:attribute"/>

		<xsl:apply-templates select="xsd:attributeGroup"/>
		<xsl:apply-templates select="xsd:complexType|xsd:element|xsd:group"/>
	</xsc:schema>

</xsl:template>

<!--  copy all elements by default   -->
<xsl:template match="*">
	<xsl:if test="not(ancestor::xsd:documenation or ancestor::xsd:appinfo)">
		<xsl:message>Warning: unrecognized element <xsl:value-of select="name()"/>.</xsl:message>
	</xsl:if>

	<xsl:element name="{local-name()}" namespace="{namespace-uri()}">
		<xsl:apply-templates select="@*|*|text()"/>
	</xsl:element>
</xsl:template>

<xsl:template match="xsd:attributeGroup">
	<xsl:if test="not(@name or @ref)">
		<xsl:message terminate="yes">Fatal Error: attributeGroup does not have a name or a ref attribute.</xsl:message>
	</xsl:if>

	<xsl:if test="parent::xsd:schema and not(@name)">
		<xsl:message terminate="yes">Fatal Error: attributeGroup that is immediate child of schema element does not have a name attribute.</xsl:message>
	</xsl:if>

	<xsl:if test="not(parent::xsd:schema) and @name">
		<xsl:message terminate="yes">Fatal Error: attributeGroup that is not immediate child of schema element has a name attribute.</xsl:message>
	</xsl:if>

	<xsl:if test="@name and @ref">
		<xsl:message terminate="yes">Fatal Error: attributeGroup has both a name and a ref attribute.</xsl:message>
	</xsl:if>

	<xsc:attributeGroup>
		<xsl:apply-templates select="@*|xsd:attribute|xsd:attributeGroup|xsd:annotation"/>
	</xsc:attributeGroup>
</xsl:template>

<xsl:template match="xsd:appinfo">
	<xsc:appinfo>
		<xsl:apply-templates select="@*|*"/>
	</xsc:appinfo>
</xsl:template>

<xsl:template match="xsd:documentation">
	<xsc:documentation>
		<xsl:apply-templates select="@*|*|text()"/>
	</xsc:documentation>
</xsl:template>


<xsl:template match="xsd:annotation">
	<xsc:annotation>
		<xsl:apply-templates select="xsd:appinfo|xsd:documentation"/>
	</xsc:annotation>
</xsl:template>

<xsl:template match="xsd:enumeration">
	<xsl:if test="not(@value)"><xsl:message terminate="yes">Fatal Error: The value element is required for an enumeration.</xsl:message></xsl:if>
	<xsc:enumeration value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:enumeration>
</xsl:template>

<xsl:template match="xsd:pattern">
	<xsc:pattern value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:pattern>
</xsl:template>

<xsl:template match="xsd:minInclusive">
	<xsc:minInclusive value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:minInclusive>
</xsl:template>

<xsl:template match="xsd:minExclusive">
	<xsc:minExclusive value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:minExclusive>
</xsl:template>

<xsl:template match="xsd:maxInclusive">
	<xsc:maxInclusive value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:maxInclusive>
</xsl:template>

<xsl:template match="xsd:maxExclusive">
	<xsc:maxExclusive value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:maxExclusive>
</xsl:template>

<xsl:template match="xsd:duration">
	<xsc:duration value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:duration>
</xsl:template>

<xsl:template match="xsd:period">
	<xsc:period value="{@value}">
		<xsl:apply-templates select="xsd:annotation"/>
	</xsc:period>
</xsl:template>


<!--  template for simpleType, checks that nested simpleTypes are anonymous  -->
<xsl:template match="xsd:simpleType">
	<xsl:if test="string-length(@name)!=0 and not(parent::xsd:schema)">
		<xsl:message terminate="yes">Fatal error: simpleType element with name attribute does not appear as immediate child of schema element.</xsl:message>
	</xsl:if>
	<xsl:if test="string-length(@name)=0 and parent::xsd:schema">
		<xsl:message terminate="yes">Fatal error: simpleType element without name attribute appears as immediate child of schema element.</xsl:message>
	</xsl:if>

	<xsl:if test="string-length(@name)!=0 and string-length(@base)!=0 and @name=@base and @name!='urSimpleType'">
		<xsl:message terminate="yes">Fatal error: simpleType has name and base attributes with the same value, <xsl:value-of select="@name"/>.</xsl:message>
	</xsl:if>

	<xsc:simpleType>
		<xsl:apply-templates select="@*|*"/>
	</xsc:simpleType>
</xsl:template>

<xsl:template match="xsd:complexType">
	<xsl:if test="string-length(@name)!=0 and not(parent::xsd:schema)">
		<xsl:message terminate="yes">Fatal error: complexType with name attribute does not appear as immediate child of schema element.</xsl:message>
	</xsl:if>
	<xsl:if test="string-length(@name)=0 and parent::xsd:schema">
		<xsl:message terminate="yes">Fatal error: complexType element without name attribute appears as immediate child of schema element.</xsl:message>
	</xsl:if>

	<!--  this only detects the grossest circular dependencies  -->
	<xsl:if test="string-length(@name)!=0 and string-length(@base)!=0 and @name=@base">
		<xsl:message terminate="yes">Fatal error: complexType has name and base attributes with the same value, <xsl:value-of select="@name"/>.</xsl:message>
	</xsl:if>

		<!--  if derived by restriction, don't allow some things going on   -->
	<xsl:if test="@derivedBy='restriction'">
		<xsl:if test="not(@base)">
			<xsl:message terminate="yes">Fatal Error: base attribute must be specified when restricting complexType</xsl:message>
		</xsl:if>

		<xsl:for-each select="xsd:attributeGroup">
			<xsl:message terminate="yes">Fatal Error: attributeGroup not supported in complexType restriction.</xsl:message>
		</xsl:for-each>
	</xsl:if>

	<xsc:complexType>
		<xsl:apply-templates select="@abstract"/>
		<xsl:apply-templates select="@base"/>
		<xsl:apply-templates select="@content"/>
		<xsl:apply-templates select="@name"/>

		<xsl:if test="parent::xsd:schema">
			<xsl:call-template name="expand-final"/>
			<xsl:call-template name="expand-block"/>
		</xsl:if>

		<xsl:choose>
			<xsl:when test="@derivedBy='restriction'">
				<xsl:apply-templates select="xsd:annotation"/>
				<xsc:restriction>
					<xsl:apply-templates select="*[name()!='xsd:annotation']"/>
				</xsc:restriction>
			</xsl:when>

			<xsl:otherwise>
				<xsl:apply-templates select="*"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsc:complexType>

</xsl:template>

<!--  express default and fixed as an valueUse and value attribute  -->
<xsl:template name="expand-default-fixed">
	<xsl:choose>
		<xsl:when test="@fixed">
			<xsl:if test="@default">
				<xsl:message>Warning: both default and fixed attributes specified for element or attribute <xsl:value-of select="@name"/>, fixed value used.</xsl:message>
			</xsl:if>
			<xsl:attribute name="use">fixed</xsl:attribute>
			<xsl:attribute name="value"><xsl:value-of select="@fixed"/></xsl:attribute>
		</xsl:when>

		<xsl:when test="@default">
			<xsl:attribute name="use">default</xsl:attribute>
			<xsl:attribute name="value"><xsl:value-of select="@default"/></xsl:attribute>
		</xsl:when>

		<xsl:otherwise>
			<!--  optional is probably not the best name for it, but it is used for attributes  -->
			<xsl:attribute name="use">optional</xsl:attribute>
		</xsl:otherwise>

	</xsl:choose>
</xsl:template>


<xsl:template name="expand-minmax">
	<xsl:param name="minDefault"/>

	<xsl:choose>
		<xsl:when test="string-length(@minOccurs) &gt; 0">
			<xsl:choose>
				<xsl:when test="number(@minOccurs) &gt; -0.1">
					<xsl:variable name="minOccurs" select="round(@minOccurs)"/>
					<xsl:attribute name="minOccurs"><xsl:value-of select="$minOccurs"/></xsl:attribute>
					<xsl:call-template name="expand-max">
						<xsl:with-param name="maxDefault">1</xsl:with-param>
						<xsl:with-param name="minOccurs" select="$minOccurs"/>
					</xsl:call-template>
				</xsl:when>

				<xsl:otherwise>
					<xsl:message terminate="yes">Fatal Error: minOccurs value (xsl:value-of select="@minOccurs"/>) is not a non-negative number.</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>

		<xsl:otherwise>
			<xsl:attribute name="minOccurs"><xsl:value-of select="$minDefault"/></xsl:attribute>
			<xsl:call-template name="expand-max">
				<xsl:with-param name="maxDefault" select="$minDefault"/>
				<xsl:with-param name="minOccurs" select="$minDefault"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<!--  expand-max ignores maxDefault values less than 1   -->
<xsl:template name="expand-max">
	<xsl:param name="minOccurs"/>
	<xsl:param name="maxDefault"/>

	<xsl:choose>
		<!--  if maxOccur is unbounded, write a dummy value for maxOccurs and set the unbounded attribute  -->
		<xsl:when test="@maxOccurs='unbounded'">
			<xsl:attribute name="unbounded">true</xsl:attribute>
			<xsl:attribute name="maxOccurs"><xsl:value-of select="$maxDefault"/></xsl:attribute>
		</xsl:when>

		<!--  when maxOccurs is present   -->
		<xsl:when test="string-length(@maxOccurs) &gt; 0">
			<!--  then set unbounded to false  -->
			<xsl:attribute name="unbounded">false</xsl:attribute>
			<xsl:choose>
				<!--  if it is a non-negative number  -->
				<xsl:when test="number(@maxOccurs) &gt; -0.1">
					<!--   maxOccurs is the closest integer   -->
					<xsl:variable name="maxOccurs" select="round(@maxOccurs)"/>

					<!--  max sure maxOccurs is greater than minOccurs   -->
					<xsl:if test="minOccurs &gt; maxOccurs + 0.1">
						<xsl:message terminate="yes">Fatal Error: minOccurs (<xsl:value-of select="$minOccurs"/>) is greater than maxOccurs (<xsl:value-of select="$maxOccurs"/>).</xsl:message>
					</xsl:if>

					<xsl:attribute name="maxOccurs"><xsl:value-of select="$maxOccurs"/></xsl:attribute>
				</xsl:when>

				<!--  otherwise stop processing    -->
				<xsl:otherwise>
					<xsl:message terminate="yes">Fatal Error: maxOccurs value (<xsl:value-of select="@maxOccurs"/>) is not a non-negative number.</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>

		<!--  otherwise use the default   -->
		<xsl:otherwise>
			<xsl:attribute name="unbounded">false</xsl:attribute>
			<!-- don't accept a default value less than 1  -->
			<xsl:choose>
				<xsl:when test="$maxDefault &lt; 1">
					<xsl:attribute name="maxOccurs">1</xsl:attribute>
				</xsl:when>

				<xsl:otherwise>
					<xsl:attribute name="maxOccurs"><xsl:value-of select="$maxDefault"/></xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="xsd:group">
	<xsl:if test="@name and not(parent::xsd:schema)">
		<xsl:message terminate="yes">Fatal error: group element with name attribute does not appear as immediate child of schema element.</xsl:message>
	</xsl:if>
	<xsl:if test="not(@name) and parent::xsd:schema">
		<xsl:message terminate="yes">Fatal error: group element without name attribute appears as immediate child of schema element.</xsl:message>
	</xsl:if>

	<xsl:if test="@ref and @name">
		<xsl:message terminate="yes">Fatal error: group element may not have both name and ref attribute.</xsl:message>
	</xsl:if>

	<xsc:group>
		<xsl:call-template name="expand-minmax">
			<xsl:with-param name="minDefault">1</xsl:with-param>
			<xsl:with-param name="maxDefault">1</xsl:with-param>
		</xsl:call-template>

		<xsl:apply-templates select="@name|@ref|*"/>
	</xsc:group>
</xsl:template>

<xsl:template match="xsd:choice">
	<xsc:group compositor="choice">
		<xsl:call-template name="expand-minmax">
			<xsl:with-param name="minDefault">1</xsl:with-param>
		</xsl:call-template>
		<xsl:apply-templates select="xsd:element|xsd:group|xsd:choice|xsd:sequence|xsd:any"/>
	</xsc:group>
</xsl:template>

<xsl:template match="xsd:all">
	<xsc:group compositor="all">
		<xsl:call-template name="expand-minmax">
			<xsl:with-param name="minDefault">1</xsl:with-param>
		</xsl:call-template>
		<xsl:apply-templates select="xsd:element|xsd:group|xsd:choice|xsd:sequence|xsd:any"/>
	</xsc:group>
</xsl:template>

<xsl:template match="xsd:sequence">
	<xsc:group compositor="sequence">
		<xsl:call-template name="expand-minmax">
			<xsl:with-param name="minDefault">1</xsl:with-param>
		</xsl:call-template>
		<xsl:apply-templates select="xsd:element|xsd:group|xsd:choice|xsd:sequence|xsd:any"/>
	</xsc:group>
</xsl:template>



<!--  recursive parsing of the final string   -->
<xsl:template name="expand-final-part3">
	<xsl:param name="final"/>

	<xsl:variable name="before-space" select="substring-before($final,' ')"/>

	<xsl:choose>
		<xsl:when test="$before-space">
			<xsl:call-template name="expand-final-part3">
				<xsl:with-param name="final" select="$before-space"/>
			</xsl:call-template>

			<xsl:call-template name="expand-final-part3">
				<xsl:with-param name="final" select="substring-after($final,' ')"/>
			</xsl:call-template>
		</xsl:when>

		<xsl:otherwise>
			<xsl:choose>
				<xsl:when test="$final='restriction'">
					<xsl:attribute name="final-restriction">true</xsl:attribute>
				</xsl:when>

				<xsl:when test="$final='extension'">
					<xsl:attribute name="final-extension">true</xsl:attribute>
				</xsl:when>

				<xsl:otherwise>
					<xsl:message terminate="yes">Fatal Error: <xsl:value-of select="$final"/> is not valid for a final attribute.</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<xsl:template name="expand-final-part2">
	<xsl:param name="final"/>

	<xsl:choose>
		<!-- if no final setting is specified, then any type of derivation is allowed  -->
		<xsl:when test="not($final) or string-length($final)=0">
			<xsl:attribute name="final-restriction">false</xsl:attribute>
			<xsl:attribute name="final-extension">false</xsl:attribute>
		</xsl:when>

		<!-- if final setting #all, then no type of derivation is allowed  -->
		<xsl:when test="$final='#all'">
			<xsl:attribute name="final-restriction">true</xsl:attribute>
			<xsl:attribute name="final-extension">true</xsl:attribute>
		</xsl:when>

		<xsl:otherwise>
			<!--  set  both attribute to false by default  -->
			<xsl:attribute name="final-restriction">false</xsl:attribute>
			<xsl:attribute name="final-extension">false</xsl:attribute>

			<!--  recursively parse the final name  -->
			<xsl:call-template name="expand-final-part3">
				<xsl:with-param name="final" select="$final"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="expand-final">
	<xsl:choose>
		<xsl:when test="@final">
			<xsl:call-template name="expand-final-part2">
				<xsl:with-param name="final" select="@final"/>
			</xsl:call-template>
		</xsl:when>

		<xsl:otherwise>
			<xsl:call-template name="expand-final-part2">
				<xsl:with-param name="final" select="parent::xsd:schema/@finalDefault"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!--  recursive parsing of the final string   -->
<xsl:template name="expand-block-part3">
	<xsl:param name="block"/>

	<xsl:variable name="before-space" select="substring-before($block,' ')"/>

	<xsl:choose>
		<xsl:when test="$before-space">
			<xsl:call-template name="expand-block-part3">
				<xsl:with-param name="block" select="$before-space"/>
			</xsl:call-template>

			<xsl:call-template name="expand-block-part3">
				<xsl:with-param name="block" select="substring-after($block,' ')"/>
			</xsl:call-template>
		</xsl:when>

		<xsl:otherwise>
			<xsl:choose>
				<xsl:when test="$block='restriction'">
					<xsl:attribute name="block-restriction">true</xsl:attribute>
				</xsl:when>

				<xsl:when test="$block='extension'">
					<xsl:attribute name="block-extension">true</xsl:attribute>
				</xsl:when>

				<xsl:when test="$block='equivClass'">
					<xsl:attribute name="block-equivClass">true</xsl:attribute>
				</xsl:when>

				<xsl:otherwise>
					<xsl:message terminate="yes">Fatal Error: <xsl:value-of select="$block"/> is not valid for a block attribute.</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<xsl:template name="expand-block-part2">
	<xsl:param name="block"/>

	<xsl:choose>
		<!-- if no block setting is specified, then any type of derivation is allowed  -->
		<xsl:when test="not($block) or string-length($block)=0">
			<xsl:attribute name="block-restriction">false</xsl:attribute>
			<xsl:attribute name="block-extension">false</xsl:attribute>
			<xsl:attribute name="block-equivClass">false</xsl:attribute>
		</xsl:when>

		<!-- if block setting #all, then no type of derivation is allowed  -->
		<xsl:when test="$block='#all'">
			<xsl:attribute name="block-restriction">true</xsl:attribute>
			<xsl:attribute name="block-extension">true</xsl:attribute>
			<xsl:attribute name="block-equivClass">true</xsl:attribute>
		</xsl:when>

		<xsl:otherwise>
			<!--  set  both attribute to false by default  -->
			<xsl:attribute name="block-restriction">false</xsl:attribute>
			<xsl:attribute name="block-extension">false</xsl:attribute>
			<xsl:attribute name="block-extension">false</xsl:attribute>

			<!--  recursively parse the block name  -->
			<xsl:call-template name="expand-block-part3">
				<xsl:with-param name="block" select="$block"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="expand-block">
	<xsl:choose>
		<xsl:when test="@block">
			<xsl:call-template name="expand-block-part2">
				<xsl:with-param name="block" select="@block"/>
			</xsl:call-template>
		</xsl:when>

		<xsl:otherwise>
			<xsl:call-template name="expand-block-part2">
				<xsl:with-param name="block" select="parent::xsd:schema/@blockDefault"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="xsd:element">
	<xsl:if test="@type and (xsd:complexType or xsd:simpleType)">
		<xsl:message terminate="yes">Fatal Error: May not specify both a type attribute and contain a complexType or simpleType element.</xsl:message>
	</xsl:if>

	<xsl:if test="@name and @ref">
		<xsl:message terminate="yes">Fatal Error: May not specify both a name and ref attribute on an xsd:element.</xsl:message>
	</xsl:if>

	<xsl:choose>
		<xsl:when test="parent::xsd:schema">
			<xsl:if test="not(@name)">
				<xsl:message terminate="yes">Fatal Error: Only named xsd:elements may appear as immediate children of xsc:schema.</xsl:message>
			</xsl:if>
		</xsl:when>

		<xsl:otherwise>
			<xsl:if test="@abstract='true' or @abstract='1'">
				<xsl:message terminate="yes">Fatal Error: xsd:elements with abstract='true' may only appear as immediate children of xsc:schema.</xsl:message>
			</xsl:if>				

			<xsl:if test="string-length(@final) &gt; 0 ">
				<xsl:message terminate="yes">Fatal Error: Nested elements should not have a final attribute.</xsl:message>
			</xsl:if>

			<xsl:if test="string-length(@equivClass) &gt; 0 ">
				<xsl:message terminate="yes">Fatal Error: Nested elements should not have an equivClass attribute.</xsl:message>
			</xsl:if>

			<xsl:if test="string-length(@block) &gt; 0 ">
				<xsl:message terminate="yes">Fatal Error: Nested elements should not have a block attribute.</xsl:message>
			</xsl:if>
		</xsl:otherwise>
	</xsl:choose>



	<xsc:element>
		<!--   top level elements do not need to expand min and max   -->
		<xsl:if test="not(parent::xsd:schema)">
			<xsl:call-template name="expand-minmax">
				<xsl:with-param name="minDefault">1</xsl:with-param>
				<xsl:with-param name="maxDefault">1</xsl:with-param>
			</xsl:call-template>
		</xsl:if>

		<!--  tried to do this as one long select but it failed   -->
		<xsl:apply-templates select="@abstract"/>
		<xsl:apply-templates select="@equivClass"/>
		<xsl:apply-templates select="@form"/>
		<xsl:apply-templates select="@name"/>
		<xsl:apply-templates select="@nullable"/>
		<xsl:apply-templates select="@ref"/>
		<xsl:apply-templates select="@type"/>

		<!--  provide final and block based on schema defaults  -->
		<xsl:if test="parent::xsd:schema">
			<xsl:call-template name="expand-final"/>
			<xsl:call-template name="expand-block"/>
			<xsl:call-template name="expand-default-fixed"/>
		</xsl:if>

		<xsl:apply-templates select="xsd:annotation|xsd:simpleType|xsd:complexType|xsd:unique|xsd:key|xsd:keyref"/>

	</xsc:element>
</xsl:template>

<!--  temporary stripping of has-facet and has-property since they are not appropriately namespaced  -->
<xsl:template match="has-facet"/>
<xsl:template match="has-property"/>
<xsl:template match="xsd:has-facet"/>
<xsl:template match="xsd:has-property"/>

<xsl:template match="xsd:attribute">
	<xsl:if test="string-length(@type)!=0 and xsd:simpleType">
		<xsl:message terminate="yes">Fatal Error: Attribute <xsl:value-of select="@name"/> May not specify both a type attribute and contain a simpleType element.</xsl:message>
	</xsl:if>

	<xsl:if test="string-length(@ref)!=0 and (xsd:simpleType or @type)">
		<xsl:message terminate="yes">Fatal Error: Reference to global attribute <xsl:value-of select="@ref"/> may not specify a type attribute or simpleType child element.</xsl:message>
	</xsl:if>

	<xsc:attribute>

		<!--  if the form attribute was not specified, then use value of attributeFormDefault   -->
		<xsl:choose>
			
			<!--  if we are an immediate descendent of the schema, we should be qualified period.  -->
			<xsl:when test="parent::xsd:schema">
				<xsl:if test="@form!='qualified'">
					<xsl:choose>
						<xsl:when test="string-length(@form)=0 and ancestor::xsd:schema[@attributeFormDefault]!='qualified'">
							<xsl:message>Warning: Top level attribute <xsl:value-of select="@name"/> was processed as qualified in conflict with its parent attributeFormDefault value.</xsl:message>
						</xsl:when>

						<xsl:when test="@form='unqualified'">
							<xsl:message>Warning: Top level attribute <xsl:value-of select="@name"/> was processed as qualified in conflict with its form attribute.</xsl:message>
						</xsl:when>
					</xsl:choose>
				</xsl:if>
				<xsl:attribute name="form">qualified</xsl:attribute>
			</xsl:when>


			<xsl:when test="string-length(@form)!=0">
				<xsl:attribute name="form"><xsl:value-of select="@form"/></xsl:attribute>
			</xsl:when>

			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="ancestor::xsd:schema/@attributeFormDefault='qualified'">
						<xsl:attribute name="form">qualified</xsl:attribute>
					</xsl:when>

					<xsl:otherwise>
						<xsl:attribute name="form">unqualified</xsl:attribute>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:apply-templates select="@name"/>
		<xsl:apply-templates select="@ref"/>
		<xsl:apply-templates select="@type"/>

		<!--  fixed and value are really orthogonal to
		           required, optional and prohibited   -->
		<xsl:choose>
			<xsl:when test="@use='fixed'">
				<xsl:attribute name="fixed"><xsl:value-of select="@value"/></xsl:attribute>
				<!--  don't generate a use=optional since it could be interpreted as a restriction  -->
			</xsl:when>

			<xsl:when test="@use='default'">
				<xsl:attribute name="default"><xsl:value-of select="@value"/></xsl:attribute>
				<!--  don't generate a use=optional since it could be interpreted as a restriction  -->
			</xsl:when>

			<xsl:otherwise>
				<xsl:apply-templates select="@use"/>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:apply-templates select="xsd:annotation|xsd:simpleType"/>

	</xsc:attribute>

</xsl:template>



<!--  copy all attributes by default   -->
<xsl:template match="@*">
	<xsl:attribute name="{name()}"><xsl:value-of select="."/></xsl:attribute>
</xsl:template>

<!--   import is suppressed in the initial fragment  -->
<xsl:template match="xsd:import">
	<xsc:namespace uri="{@namespace}">
		<xsl:choose>
			<xsl:when test="@xsc:prefix">
				<xsc:prefix><xsl:value-of select="@xsc:prefix"/></xsc:prefix>
			</xsl:when>

			<xsl:otherwise>
				<xsl:message>Information: Namespace <xsl:value-of select="@namespace"/> did not have a prefix associated with it, please add a xsc:prefix attribute to allow resolution of qualified names.</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsc:namespace>
</xsl:template>

<!--  this named template processed xsd:include directives and
          traps include nesting greater than 10 levels  -->
<xsl:template name="process-inclusion">
	<xsl:param name="include-depth"/>

	<xsl:variable name="includedDoc" select="document(@schemaLocation,.)"/>
	<xsl:choose>
		<xsl:when test="$includedDoc">
			<!--   we add the compiled inclusion tag to give any declared namespaces
		          a place to live   -->
			<xsl:for-each select="$includedDoc/*/xsd:include">
				<xsl:choose>
					<xsl:when test="number($include-depth)+1 &gt; 10">
						<xsl:message terminate="yes">Fatal error: Nested included depth greater than maximum.</xsl:message>
					</xsl:when>

					<xsl:otherwise>
						<xsl:call-template name="process-inclusion">
							<xsl:with-param name="include-depth"><xsl:value-of select="number($include-depth)+1"/></xsl:with-param>
						</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:for-each>

			<!--  it just gets nasty working out the URL for an import in an include
			          so were are ignoring them for now   -->
			<xsl:for-each select="$includedDoc/*/xsd:import">
				<xsl:message>Warning: import for namespace <xsl:value-of select="@namespace"/> appearing in included resource was ignored.</xsl:message>
			</xsl:for-each>

			<xsl:apply-templates select="$includedDoc/*/*[name()!='xsd:include' and name()!='xsd:annotation' and name()!='xsd:import']"/>
			<xsl:variable name="localElements" select="$includedDoc/*/*/descendant::xsd:element[@name]"/>
			<xsl:if test="$localElements">
				<xsl:message>Information: include resource <xsl:value-of select="@schemaLocation"/> contains <xsl:value-of select="count($localElements)"/> locally scope elements that may not be correctly represented in generated DTDs.</xsl:message>
			</xsl:if>
		</xsl:when>

		<!--  node set was empty, emit fatal error  -->
		<xsl:otherwise>
			<xsl:message terminate="yes">Fatal error: Included file <xsl:value-of select="@schemaLocation"/> was not found.</xsl:message>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="xsd:include">
	<xsl:call-template name="process-inclusion">
		<xsl:with-param name="include-depth">0</xsl:with-param>
	</xsl:call-template>
</xsl:template>

<xsl:template name="process-import">
	<xsl:param name="namespace"/>
	<xsl:param name="importedSchema"/>
	<xsl:param name="blocked-namespaces"/>
	<xsl:param name="parent-namespace"/>
	<xsl:param name="implied-namespaces"/>

	<xsl:choose>
		<xsl:when test="not($importedSchema)">
			<xsl:message terminate="yes">Fatal error: Imported schema not found at <xsl:value-of select="@schemaLocation"/></xsl:message>
		</xsl:when>

		<xsl:when test="not($importedSchema/xsd:schema)">
			<xsl:message terminate="yes">Fatal error: Imported schema <xsl:value-of select="@schemaLocation"/> document element is not schema.</xsl:message>
		</xsl:when>

		<xsl:when test="$importedSchema/xsd:schema/@targetNamespace!=$namespace">
			<xsl:message terminate="yes">Fatal error: Target namespace (<xsl:value-of select="$importedSchema/xsd:schema/@targetNamespace"/>) of imported schema <xsl:value-of select="@schemaLocation"/> doe not match namespace (<xsl:value-of select="$namespace"/> on import statement.
			</xsl:message>
		</xsl:when>

		<xsl:otherwise>
			<xsl:for-each select="$importedSchema/xsd:schema/xsd:import">
				<xsl:if test="not(@namespace=$blocked-namespaces or @namespace=$parent-namespace or contains($implied-namespaces,@namespace))">
					<xsl:message>Warning: A nested import was found for namespace <xsl:value-of select="@namespace"/> at <xsl:value-of select="@schemaLocation"/>. Please import this schema at the top level to provide an unambiguous location.</xsl:message>
					<xsc:import namespace="{@namespace}" schemaLocation="{@schemaLocation}"/>
				</xsl:if>
			</xsl:for-each>
			<xsl:apply-templates select="$importedSchema/xsd:schema"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>


<xsl:template match="xsd:import" mode="imports">
	<xsl:choose>
		<xsl:when test="@namespace=/xsd:schema/@targetNamespace">
			<xsl:message terminate="yes">Fatal error: cannot include schema that defines same namespace as base schema.</xsl:message>
		</xsl:when>

		<xsl:otherwise>
			<!--    namespace and prefix go in regardless   -->
			<xsc:namespace uri="{@namespace}">
				<xsl:choose>
					<xsl:when test="@xsc:prefix">
						<xsc:prefix><xsl:value-of select="@xsc:prefix"/></xsc:prefix>
					</xsl:when>

					<xsl:otherwise>
						<xsl:message>Informative: Import of namespace <xsl:value-of select="@namespace"/> did not declare a prefix with the xsc:prefix attribute.  Qualified names may not be resolvable.</xsl:message>
					</xsl:otherwise>
				</xsl:choose>
			</xsc:namespace>

			<xsl:call-template name="process-import">
				<xsl:with-param name="namespace" select="@namespace"/>
				<xsl:with-param name="importedSchema" select="document(@schemaLocation,.)"/>
				<xsl:with-param name="blocked-namespaces" select="parent::xsd:import/@namespace"/>
				<xsl:with-param name="parent-namespace" select="ancestor::xsd:schema/@targetNamespace"/>
				<xsl:with-param name="implied-namespaces">http://www.w3.org/XML/1998/namespace http://www.w3.org/1999/XMLSchema http://www.w3.org/1999/XMLSchema-instance</xsl:with-param>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="generate-namespace">
	<xsl:param name="namespace"/>
	<xsl:choose>
		<xsl:when test="$namespace = '##targetNamespace'">
			<xsc:namespace uri="{ancestor::xsd:schema[@targetNamespace]}"/>
		</xsl:when>
		
		<xsl:when test="$namespace = '##local'">
			<xsl:message terminate="yes">Fatal error: ##local namespace support not yet implemented.</xsl:message>
		</xsl:when>

		<xsl:otherwise>
			<xsc:namespace uri="{$namespace}"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<!--  recursively parses a blank separated list of namespaces
         generating xsc:namespace entries for each   -->
<xsl:template name="parse-namespaces">
	<xsl:param name="namespaces"/>
	<xsl:variable name="before" select="substring-before($namespaces,' ')"/>
	<xsl:choose>
		<xsl:when test="$before">
			<xsl:call-template name="generate-namespace">
				<xsl:with-param name="namespace" select="$before"/>
			</xsl:call-template>
			<xsl:call-template name="parse-namespaces">
				<xsl:with-param name="namespaces" select="substring-after($namespaces,' ')"/>
			</xsl:call-template>
		</xsl:when>

		<xsl:otherwise>
			<xsl:call-template name="generate-namespace">
				<xsl:with-param name="namespace" select="$namespaces"/>
			</xsl:call-template>
		</xsl:otherwise>

	</xsl:choose>
</xsl:template>				

<xsl:template name="parse-any-namespace">
	<xsl:choose>
		<xsl:when test="@namespace='##any' or string-length(@namespace)=0">
			<xsl:attribute name="any">1</xsl:attribute>
		</xsl:when>

		<xsl:when test="@namespace='##other'">
			<xsc:namespaceNor>
				<xsc:namespace uri="{ancestor::xsd:schema/@targetNamespace}"/>
			</xsc:namespaceNor>
		</xsl:when>

		<xsl:when test="@namespace='##local'">
			<xsl:message terminate="yes">Fatal Error: not sure of significance of local.</xsl:message>
		</xsl:when>
		
		<xsl:otherwise>
			<xsl:call-template name="parse-namespaces">
				<xsl:with-param name="namespaces" select="@namespace"/>
			</xsl:call-template>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>	

<xsl:template match="xsd:anyAttribute">
	<xsc:anyAttribute>
		<xsl:call-template name="parse-any-namespace"/>
	</xsc:anyAttribute>
</xsl:template>


<xsl:template name="implicit-import">
	<xsl:param name="namespace"/>
	<xsl:param name="prefix"/>
	<xsl:param name="schemaLocation"/>

	<!--  if the namespace isn't explicitly included (or being defined)   -->
	<xsl:if test="not(import[@uri=$namespace]) and not(@targetNamespace=$namespace)">
		<xsl:call-template name="process-import">
			<xsl:with-param name="namespace" select="$namespace"/>
			<xsl:with-param name="prefix" select="$prefix"/>
			<xsl:with-param name="importedSchema" select="document($schemaLocation)"/>
			<xsl:with-param name="blocked-namespaces" select="xsd:import/@namespace"/>
			<xsl:with-param name="parent-namespace" select="@targetNamespace"/>
			<xsl:with-param name="implied-namespaces">http://www.w3.org/XML/1998/namespace http://www.w3.org/1999/XMLSchema http://www.w3.org/1999/XMLSchema-instance</xsl:with-param>
		</xsl:call-template>
	</xsl:if>
</xsl:template>

<xsl:template match="xsd:key">
	<xsc:key>
		<xsl:apply-templates select="@*|*"/>
	</xsc:key>
</xsl:template>

<xsl:template match="xsd:unique">
	<xsc:unique>
		<xsl:apply-templates select="@*|*"/>
	</xsc:unique>
</xsl:template>

<xsl:template match="xsd:keyref">
	<xsc:keyref>
		<xsl:apply-templates select="@*|*"/>
	</xsc:keyref>
</xsl:template>

<xsl:template match="xsd:selector">
	<xsc:selector><xsl:value-of select="."/></xsc:selector>
</xsl:template>

<xsl:template match="xsd:field">
	<xsc:field><xsl:value-of select="."/></xsc:field>
</xsl:template>


<xsl:template match="xsd:scale">
	<xsc:scale><xsl:apply-templates/></xsc:scale>
</xsl:template>

<xsl:template match="xsd:notation">
	<xsc:notation><xsl:apply-templates/></xsc:notation>
</xsl:template>

<xsl:template match="xsd:any">
	<xsc:any>
		<xsl:call-template name="expand-minmax">
			<xsl:with-param name="minDefault">1</xsl:with-param>
		</xsl:call-template>

		<xsl:apply-templates select="xsd:annotation"/>

		<xsl:call-template name="parse-any-namespace"/>
	</xsc:any>
</xsl:template>


</xsl:transform>